#!/usr/bin/perl
########################################################################
# blanczos64.c
# Copyright 2004, Chris Monico.
# Many fixes and improvements have been
# contributed by Greg Childers, Brian Gladman, Kamada Makoto and others.
#
#   This file is part of GGNFS.
#   GGNFS is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   GGNFS is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with GGNFS; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
########################################################################

# The path where the binaries are:
$GGNFS_BIN_PATH="/home/cmonico/Public/src/ggnfs/src";


$FORCECC=1;
$SAVEPAIRS=0;
$CLEANUP=0;
$PROMPTS=0;
$CHECK_BINARIES=1;

# Run the binaries at low priority?
#$NICE="nice -n 19 ";

# Experimental! You might try 2 if this gives you problems.
$LARGEP=3;

# The more extra FF's there are, the smaller the resulting pruned
# matrix will be (upto a certain point). Requiring the initial matrix
# to have an extra 10% columns is pretty reasonable.
$minextraFF=0.1;

# This should be in (0,1). Lower ==> smaller but denser pruned matrix.
$matWtFactor=0.25;

if($^O ne "MSWin32") {
  $CAT="cat";
  $GZIP="gzip";
  $EXEC_SUFFIX="";
}
else {
  $CAT=$GGNFS_BIN_PATH."/cat.exe";
  $GZIP=$GGNFS_BIN_PATH."/gzip.exe";
  $EXEC_SUFFIX=".exe";
}

$LATSIEVER=$GGNFS_BIN_PATH."/gnfs-lasieve4I12e".$EXEC_SUFFIX;
#$LATSIEVER=$GGNFS_BIN_PATH."/gnfs-lasieve4I13e".$EXEC_SUFFIX;
#$LATSIEVER=$GGNFS_BIN_PATH."/gnfs-lasieve4I14e".$EXEC_SUFFIX;

###################################################################
# FORCECC:
#   To force a count on the number of full relations, set this to 1.
#   The criteria by which GGNFS tries to guess is still pretty lousy,
#   so you may need to do this once in a while for a sanity check.
#   But if you are concerned about how much time is being wasted
#   counting cycles, set it to zero.
# SAVEPAIRS:
#   If this is 1, the script will save all of the siever output in a gzipped file.
# CLEANUP:
#   If this is 1, the script will clean up after itself.
# LATSIEVER:
#   There are three lattice siever binaries to choose from, depending
#   on the size of the number being factored.
#   12e: GNFS numbers upto about 110 digits (SNFS 150).
#   13e: GNFS numbers upto around 130-140 (SNFS 180).
#   14e: Larger.
####################################################################
# This script can handle externally generated relations. Simply place
# them in this directory in a file called "spairs.add", and the script
# will process them on its next procrels pass.
####################################################################

####################################################################
###### Probably nothing below here should need to be changed. ######
####################################################################
$MAKEFB=$GGNFS_BIN_PATH."/makefb".$EXEC_SUFFIX;
$PROCRELS=$GGNFS_BIN_PATH."/procrels".$EXEC_SUFFIX;
$MATSOLVE=$GGNFS_BIN_PATH."/matsolve".$EXEC_SUFFIX;
$SQRT=$GGNFS_BIN_PATH."/sqrt".$EXEC_SUFFIX;
$PLOT=$GGNFS_BIN_PATH."/autogplot.sh";

$DEPFILE="deps";
$DEPINF="depinf";
$RELSBIN="rels.bin";
$LOGFILE="ggnfs.log";
$LARGEPRIMES="-".$LARGEP."p";

# This file is used by this script to detect
# changes in parameter settings.
$PARAMFILE=".params";

# This is to tell the lattice siever where to dump the next special-q
# if it's interrupted, for example, with CTRL-C.
$PNUM=3141;


###########################################################
sub sigDie {
  die "Signal caught. Terminating...\n";
}

######################################################
sub changeParams {
# Change the factor base sizes for a factorization
# which has already been started. This is done by
# dumping the raw siever output from the rels.bin files,
# deleting the rels.bin files, re-setting the parameters,
# and reprocessing all of the relations.
  open(LF, ">>$LOGFILE");
  print LF "*** factLat.pl: Parameters changed\n";
  close(LF);

  print "---- changeParams ----\n";
  print "Dumping relations...\n";
  $res=system("$NICE \"$PROCRELS\" -fb $NAME.fb -prel $RELSBIN -dump");
  die "Return value $res. Terminating...\n" if ($res);
  unlink <$RELSBIN*>, <cols*>, <deps*>, 'factor.easy', <lpindex*>;
  unlink <$NAME.job.afb.*>;

  print "Making new factor base files...\n";
  $cmd="$NICE \"$MAKEFB\" -rl $RLIM -al $ALIM -lpbr $LPBR -lpba $LPBA $LARGEPRIMES -of $NAME.fb -if $NAME.poly";
  $res=system($cmd);
  die "Return value $res. Terminating...\n" if ($res);

  print "Reprocessing siever output...\n";
  $i=0;
  while (-e "spairs.dump.$i") {
    $res=system("$NICE \"$PROCRELS\" -fb $NAME.fb -prel $RELSBIN -newrel spairs.dump.$i");
    $i++;
  }
  # Update the paramfile, used by this script to detect changes
  # in parameter settings.
  open(INFO, ">$PARAMFILE");
  print INFO "rlim: $RLIM\n"; 
  print INFO "alim: $ALIM\n"; 
  print INFO "lpbr: $LPBR\n"; 
  print INFO "lpba: $LPBA\n"; 
  close(INFO);
} 
  
###########################################################
sub checkParamFile {
# Check two things:
# (1) Does a $PARAMFILE exist? If not, create it.
# (2) Are the values in $PARAMFILE the same as the current
#     values? If not, call changeParams to sync everything up.
  if (!(-e $PARAMFILE)) {
    print "Creating param file to detect parameter changes...\n";
    open(INFO, ">$PARAMFILE");
    print INFO "rlim: $RLIM\n"; 
    print INFO "alim: $ALIM\n"; 
    print INFO "lpbr: $LPBR\n"; 
    print INFO "lpba: $LPBA\n"; 
    close(INFO);
    return ;
  }
  # Okay - it exists. We should check it to see if the
  # parameters are the same as the current ones.
  open(INFO, $PARAMFILE);
  my @lastParams=<INFO>;
  close(INFO);
  my @TMP=grep(/rlim:/, @lastParams);
  my $OLDRLIM=@TMP[0];
  $OLDRLIM =~ s/.*rlim://;
  @TMP=grep(/alim:/, @lastParams);
  my $OLDALIM=@TMP[0];
  $OLDALIM =~ s/.*alim://;
  @TMP=grep(/lpbr:/, @lastParams);
  my $OLDLPBR=@TMP[0];
  $OLDLPBR =~ s/.*lpbr://;
  @TMP=grep(/lpba:/, @lastParams);
  my $OLDLPBA=@TMP[0];
  $OLDLPBA =~ s/.*lpba://;
  if ($OLDRLIM != $RLIM || $OLDALIM != $ALIM ||
      $OLDLPBR != $LPBR || $OLDLPBA != $LPBA) {
    print "==============================\n";
    print "Parameter change detected - handling with changeParams()...\n";
    changeParams;
  } else {
    print "No parameter change detected. Resuming.\n";
  }
}

###########################################################
sub plotLP {
  if ($CHECK_BINARIES) {
    return unless (-x $PLOT);
  }
  open(OUTF, ">.lprels") || return;
  print OUTF "0, 0\n";
  open(OUTRELS, ">.rels") || return;
  print OUTRELS "0, 0\n";
  open(LOG, $LOGFILE) || return;
  while ($_ = <LOG>) {
    chop;
    tr/a-z/A-Z/; # Convert to upper case.
    if ( /LARGEPRIMES/) {
      s/\[.*]\s*//;  # Strip out the date.
      s/(LARGEPRIMES: |RELATIONS: )//g;  # Strip out the labels.
      s/\s//g; # Remove whitespace.
      split(/,/);
      $excessLP=@_[0]-@_[1];
      print OUTF "@_[1], $excessLP\n";
    } elsif (/FINALFF/) {
      s/\[.*]\s*//;  # Strip out the date.
      s/(RELS:|INITIALFF:\d*,|FINALFF:)//g;  # Strip out the labels.
      s/\s//g; # Remove whitespace.
      split(/,/);
      print OUTRELS "@_[0], @_[1]\n";
    }
  }
  close(OUTF);
  close(OUTRELS);
  close(LOG);
  $ENV{'XAXIS'}='Total relations';
  $ENV{'YAXIS'}="";
  $cmd="\"$PLOT\""." xprimes.jpg 'ExcessLargePrimes' .lprels";
  $res=system($cmd);
  die "Return value $res. Terminating...\n" if ($res);
  $ENV{'XAXIS'}='Total relations';
  $ENV{'YAXIS'}="Full relation-sets";
  $cmd="\"$PLOT\""." relations.jpg 'TotalFF' .rels";
  $res=system($cmd);
  die "Return value $res. Terminating...\n" if ($res);
  unlink '.lprels';
  unlink '.rels';
}
######################################################
sub checkParams {

  if ($CHECK_BINARIES) {
    $missing .= 'makefb ' unless (-x $MAKEFB);
    $missing .= 'procrels ' unless (-x $PROCRELS);
    $missing .= 'matsolve ' unless (-x $MATSOLVE);
    $missing .= 'sqrt ' unless (-x $SQRT);
    $missing .= '(lattice siever) ' unless (-x $LATSIEVER);
  }
  if ($missing) {
    print "Could not find GGNFS programs: $missing.\n";
    print "Did you set GGNFS_BIN_PATH properly in this script?\n";
    print "It is currently set to:";
    print "GGNFS_BIN_PATH=$GGNFS_BIN_PATH\n";
    exit -1;
  }
  die("Error: 'n' not supplied!\n") unless ($N);
  die("Error: 'm' not supplied!\n") unless ($M);
  die("Error: polynomial not supplied!\n") unless ($COEF[0]);
  die("Error: 'skew' not supplied!\n") unless ($SKEW);
  die("Error: 'rlim' not supplied!\n") unless ($RLIM);
  die("Error: 'alim' not supplied!\n") unless ($ALIM);
  die("Error: 'lpbr' not supplied!\n") unless ($LPBR);
  die("Error: 'lpba' not supplied!\n") unless ($LPBA);
  die("Error: 'mfbr' not supplied!\n") unless ($MFBR);
  die("Error: 'mfba' not supplied!\n") unless ($MFBA);
  die("Error: 'rlambda' not supplied!\n") unless ($RLAMBDA);
  die("Error: 'alambda' not supplied!\n") unless ($ALAMBDA);
  die("Error: 'qintsize' not supplied!\n") unless ($QSTEP);
}

######################################################
sub makeJobFile {
  # arg0 = filename
  # arg1 = first q value
  # arg2 = q range size
  if ($#_ < 2) { die("makeJobFile() : Not enough arguments!\n") };
  $FNAME=$_[0];
  unlink $FNAME;
  open(OUTF, ">$FNAME");
  print OUTF "n: $N\nm: $M\n";
  # The polynomial coefficients:
  for ($i=0; length(${COEF[$i]}) > 0; $i += 2) {
    print OUTF "${COEF[$i]} ${COEF[${i}+1]}\n";
  }
  print OUTF "skew: $SKEW\n";
  print OUTF "rlim: $RLIM\n";
  if ($ALIM > $_[1]) { 
    $sieverAL = $_[1]-1; 
    unlink <$NAME.job.afb.*>;
  }
  else { $sieverAL = $ALIM; }
  print OUTF "alim: $sieverAL\n";
  print OUTF "lpbr: $LPBR\n";
  print OUTF "lpba: $LPBA\n";
  print OUTF "mfbr: $MFBR\n";
  print OUTF "mfba: $MFBA\n";
  print OUTF "rlambda: $RLAMBDA\n";
  print OUTF "alambda: $ALAMBDA\n";
  print OUTF "q0: $_[1]\n";
  print OUTF "qintsize: $_[2]\n"; 
  close(OUTF); 
}

######################################################
sub readParams {
  $Q0=0;
  open(PF, $NAME.".poly");
  while (<PF>) {
    chop;
    s/#.*//; # Remove comments
    s/\s*//g; # Remove whitespace.
    split(/:/);
    if (@_[0] eq "n") { $N=@_[1]; }
    if (@_[0] eq "m") { $M=@_[1]; }
    if (@_[0] eq "rlim") { $RLIM=@_[1]; }
    if (@_[0] eq "alim") { $ALIM=@_[1]; }
    if (@_[0] eq "lpbr") { $LPBR=@_[1]; }
    if (@_[0] eq "lpba") { $LPBA=@_[1]; }
    if (@_[0] eq "mfbr") { $MFBR=@_[1]; }
    if (@_[0] eq "mfba") { $MFBA=@_[1]; }
    if (@_[0] eq "rlambda") { $RLAMBDA=@_[1]; }
    if (@_[0] eq "alambda") { $ALAMBDA=@_[1]; }
    if (@_[0] eq "knowndiv") { $KNOWNDIV=@_[1]; }
    if (@_[0] eq "skew") { $SKEW=@_[1]; }
    if (@_[0] eq "q0") { $Q0=@_[1]; }
    if (@_[0] eq "qintsize") { $QSTEP=@_[1]; }
    if (@_[0] =~ /c./) {
      push(@COEF, @_[0].":");
      push(@COEF, @_[1]);
    }
  }
  if ($KNOWNDIV) { $KNOWNDIV="-knowndiv ".$KNOWNDIV; }
#  if ($Q0 >= $ALIM) { $QSTART=$Q0; }
#  else { $QSTART=$ALIM; }
  if ($Q0==0) {
    $Q0 = $ALIM/2;
  }
  $QSTART=$Q0;
  print "Calling checkParamFile...\n";
  checkParamFile;
}



######################################################
sub setup {
  #########################################
  # Should we resume from an earlier run? #
  #########################################
  $resume=-1;
  if (-e $RELSBIN.".0") {
    print "It appears that an earlier attempt was interrupted in progress. Resume? (y/n) ";
    if ($PROMPTS) {
      do {
        $_=getc;
        if (($_ eq "Y") || ($_ eq "y")) { $resume=1; }
        elsif (($_ eq "N") || ($_ eq "n")) { $resume=0; }
      } while ($resume < 0);
    } else { $resume=1; }
  }

  ############################################
  # Setup the parameters for sieving ranges. #
  ############################################
  if ($resume != 1) {
    # Clean up any junk leftover from an earlier attempt.
    unlink $LOGFILE, <cols*>, <deps*>, 'factor.easy', <lpindex*>;
    unlink <rels*>, 'spairs.out', 'spairs.save.gz', $NAME.'.fb', <*.afb.0>;
    unlink '.params';

    # Was a discriminant divisor supplied?
    if ($DD) {
      open(INFO, $LOGFILE);
      print INFO "$DD\n";
      close(INFO);
    }
    # Create the factor base.
    $cmd="$NICE \"$MAKEFB\" -rl $RLIM -al $ALIM -lpbr $LPBR -lpba $LPBA $LARGEPRIMES -of $NAME.fb -if $NAME.poly";
    $res=system($cmd);
    die "Return value $res. Terminating...\n" if ($res);
    $Q0=$QSTART;
  } else {
    # Get the Q0 value from tmp.job and just restart from there.
    if (-e $JOBNAME) {
      open(INFO, $JOBNAME);
      @lastJobF=<INFO>;
      close(INFO);
      @Q0LINE=grep(/q0:/, @lastJobF);
      $Q0=@Q0LINE[0];
      $Q0 =~ s/.*q0://;
      die("Could not recover q0: field from job file $JOBNAME!\n") unless ($Q0);
    } else {
      print "File $JOBNAME does not exist. Could not determine a starting q value!\n";
      print "Please enter a starting point for the special q: ";
      open(INFO, '-');
      $Q0=<INFO>;
      close(INFO);
    }
  }
}


############### Begin here #################
$SIG{'INT'}=\&sigDie;
print 
"___________________________________________________________
|        This is the factLat.pl script for GGNFS.          |
| This program is copyright 2004, Chris Monico, and subject|
| to the terms of the GNU General Public License version 2.|
|__________________________________________________________|\n";

if ($#ARGV != 0) {
  print "USAGE: $0 <polynomial file>\n";
  exit;
}
$NAME=@ARGV[0];
$NAME =~ s/\.poly//;
print "Working with NAME=$NAME...\n";
$JOBNAME=$NAME.".job";

######################################################
# We won't take the liberty of deleting a dependency #
# file in case this script is run by accident.       #
######################################################
if (-e $DEPFILE) {
  die("File $DEPFILE already exists. Delete it and re-run for a fresh factorization");
}
die("Error: Polynomial file $NAME.poly does not exist!") unless (-e $NAME.".poly");
# Read and verify parameters for the factorization.
readParams;
checkParams;

setup;

######################################################################
## Find the minimum number of FF's to get.                          ##
## For this, we must determine the RFB,AFB sizes from the log file. ##
######################################################################
open(INFO, $LOGFILE);
while (<INFO>) {
  split;
  if (@_[2] eq "RFBsize:") { $rfbSize = @_[3]; }
  if (@_[2] eq "AFBsize:") { $afbSize = @_[3]; }
}
close(INFO);
$totalSize=$rfbSize+$afbSize+64;
$minFF=$totalSize*(1.0 + $minextraFF);
print "minimum number of FF's: $minFF\n";


####################################################
# Finally, sieve until `procrels' creates a        #
# depinf file, signalling that sieving is done and #
# we have reached the desired min # of FF's.       #
####################################################
while (!(-e $DEPINF)) {
  # This could happen if afb limit was increased during a factorization.
#  if ($Q0 < $ALIM) {
#    $Q0 = $ALIM+1;
#  }
  print "*******************************\n";
  printf("Lattice sieving q-values from q=$Q0 to %d.\n", $Q0+$QSTEP);
  # Create a job file.
  makeJobFile($JOBNAME, $Q0, $QSTEP);
  $startTime = time;

  # If we are sieving below the AFB limit, we need to delete the
  # siever's factor base file to make it create a new one. This is
  # a dirty hack - what we should do is modify Franke's code to
  # explicitly allow it.
  if ($sieverAL == ($Q0-1)) {
    unlink <$NAME.job.afb.*>;
  }
  # It's very important to call like this, so that if the user CTRL-C's,
  # or otherwise kills the process, we see it and terminate as well.
  $res=system("$NICE \"$LATSIEVER\" -k -o spairs.out -v -a $JOBNAME -n$PNUM");
  if ($res) {
    print "Return value $res. Updating job file and terminating...\n";
    open(INFO, ".last_spq$PNUM");
    $lastSPQ=<INFO>;
    chop $lastSPQ;
    close(INFO);
    unlink ".last_spq$PNUM";
    if ($lastSPQ < $Q0) {
      $lastSPQ = $Q0;
    }
    # Move the new relations so they won't be wiped on restart.
    system("\"$CAT\" spairs.out >> spairs.add");
    unlink "spairs.out";
    # And update the job file accordingly:
    makeJobFile($JOBNAME, $lastSPQ, $QSTEP);
    # Record the time to the logfile.
    $stopTime = time;
    $totalTime = $stopTime - $startTime;
#    open(INFO, ">>$LOGFILE");
#    print INFO "        LatSieveTime: $totalTime\n";
#    close(INFO);
    die "Terminating...\n";
  }

  die("Some error ocurred and no relations were found! Examing log file.\n") unless
      -e "spairs.out";

  # Are there relations coming from somewhere else which should be added in?
  if (-e "spairs.add") {
    system("\"$CAT\" spairs.add >> spairs.out");
    unlink "spairs.add";
  }

  $stopTime = time;
  $totalTime = $stopTime - $startTime;
#  open(INFO, ">>$LOGFILE");
#  print INFO "        LatSieveTime: $totalTime\n";
#  close(INFO);
  if ($FORCECC == 1) {
    $res=system("$NICE \"$PROCRELS\" -fb $NAME.fb -prel $RELSBIN -newrel spairs.out -forceCC");
    die "Return value $res. Terminating...\n" if ($res);
  } else {
    $res=system("$NICE \"$PROCRELS\" -fb $NAME.fb -prel $RELSBIN -newrel spairs.out");
    die "Return value $res. Terminating...\n" if ($res);
  }
  if ($SAVEPAIRS) {
    $res=system("$NICE \"$GZIP\" -c spairs.out >> spairs.save.gz"); 
    die "Return value $res. Terminating...\n" if ($res);
  }
  unlink "spairs.out";
  $Q0=$Q0+$QSTEP;
  plotLP;
  # Find out how many FF's there are.
  open(INFO, $LOGFILE);
  while (<INFO>) {
    chop;
    if (s/finalFF://) {
      $t=$';
    }
  }
  close(INFO);
  if ($t < $minFF) {
    # Remove the `depinf' file so we can do some more sieving.
    unlink "depinf";
  }
  unlink $TMPJOBNAME;
}

###############################
# Obviously, the matrix step. #
###############################
print "Doing matrix step...\n";
$res=system("$NICE \"$MATSOLVE\""." -wt ".$matWtFactor); 
die "Return value $res. Terminating...\n" if ($res);
die("Some error occurred and matsolve did not record dependencies.\n") unless
   (-e $DEPFILE);

#############################################
# Do as many square root jobs as needed to  #
# get the final factorization.              #
#############################################
$depnum=0;
$res=system("$NICE \"$SQRT\" $DISC -fb $NAME.fb -deps $DEPFILE -depnum $depnum $KNOWNDIV");
while ($res != 0) {
  if (($res==1) || ($res==256)) {
    print "Composite factor found. Try another dependency? (y/n) ";
    if ($PROMPTS) {
      open(INFO, '-');
      $_=<INFO>;
      close(INFO);
      if (($_ eq "N") || ($_ eq "n")) { $res=0; }
    } else { $res=1 unless ($depnum > 8); }
  }
  if ($res) {
    $depnum=$depnum+1;
    if ($depnum >= 32) {
      print "Some error has probably occurred - all dependencies have been exhausted!\n";
      exit -1;
    }
    print "Computing square root for dependency $depnum ...\n";
    $res=system("$NICE \"$SQRT\" $DISC -fb $NAME.fb -deps $DEPFILE -depnum $depnum $KNOWNDIV");
  }
}

if ($CLEANUP) {
  unlink $LOGFILE, <cols*>, <deps*>, 'factor.easy', <lpindex*>;
  unlink <rels*>, 'spairs.out', 'spairs.save.gz', $NAME.'.fb', <*.afb.0>;
  unlink "tmpdata.000";
  unlink $PARAMFILE;
}

# And gather up some stats.
$sieveT=0.0;
$relprocT=0.0;
open(INFO,$LOGFILE);
while (<INFO>) {
  chop;
  split;
  if (@_[0] eq "LatSieveTime:") { $sieveT += @_[1]; }
  if (@_[2] eq "RelProcTime:") { $relprocT += @_[3]; }
  if (@_[2] eq "BLanczosTime:") { $matT = @_[3]; }
  if (@_[2] eq "sqrtTime:") { $sqrtT = @_[3]; }
  if (/rels:/) { $rels=@_[2]." ".@_[4]; }
  if (/Initial matrix/) { s/\[.*\] Initial matrix is //; $initmat=$_; }
  if (/Matrix pruned/) { s/\[.*\] Matrix pruned to //; $prunedmat=$_; }
  if (/GGNFS-/) { $version=@_[2]; }
  if (/r.=/) {
    s/.*r.=//;
    if ((length($_) > 1) && (length($_) < length($N))) {
      push(@DIVISORS, $_);
    }
  }
}
close(INFO);
# Convert times from seconds to hours.
$sieveT /= 3600.0; $relprocT /= 3600.0;
$matT /= 3600.0;   $sqrtT /= 3600.0;
$totalT=$sieveT+$relprocT+$matT+$sqrtT;

open(STDOUT, ">summary.txt");
print "Number: $NAME\n";
print "N=$N\n";
printf("  ( %d digits)\n", length($N));
print "Divisors found:\n";
while ($_=pop(@DIVISORS)) {
  print "$_\n";
}
printf("Version: $version\n");
printf("total time: %1.2f hours.\n", $totalT);
print "Factorization parameters were as follows:\n";
open(PARS, "$NAME.poly");
while (<PARS>) { print $_; }
close(PARS);
print "Factor base limits: $RLIM/$ALIM\n";
print "Large primes per side: $LARGEP\n";
print "Large prime bits: $LPRBITS/$LPABITS\n";
print "Sieved special-q in [$QSTART, $Q0)\n";
print "Relations: $rels\n";
print "Initial matrix: $initmat\n";
print "Pruned matrix : $prunedmat\n";
printf("Total sieving time: %1.2f hours.\n", $sieveT);
printf("Total relation processing time: %1.2f hours.\n", $relprocT);
printf("Matrix solve time: %1.2f hours.\n", $matT);
printf("Time per square root: %1.2f hours.\n", $sqrtT);
printf("total time: %1.2f hours.\n", $totalT);
print " --------- CPU info (if available) ----------\n";
close(STDOUT);
if (-x '/bin/dmesg') {
  system("/bin/dmesg | grep CPU | grep stepping >> summary.txt");
  system("/bin/dmesg | grep Memory >> summary.txt");
  system("/bin/dmesg | grep -i bogomips >> summary.txt");
}
if (-x '/usr/sbin/x86info') {
  system("/usr/sbin/x86info -mhz >> summary.txt");
}
print "Factorization summary written to summary.txt.\n"

